searchNodes=[{"doc":"Creates an Ecto.Schema.embedded_schema that supplies a command with all the validation power of the Ecto.Changeset data structure. defmodule CreateAccount do use Commanded.Command, username: :string, email: :string, age: :integer def handle_validate(changeset) do changeset |&gt; validate_required([:username, :email, :age]) |&gt; validate_format(:email, ~r/@/) |&gt; validate_number(:age, greater_than: 12) end end iex&gt; CreateAccount.new(username: &quot;chris&quot;, email: &quot;chris@example.com&quot;, age: 5) #Ecto.Changeset&lt;action: nil, changes: %{age: 5, email: &quot;chris@example.com&quot;, username: &quot;chris&quot;}, errors: [age: {&quot;must be greater than %{number}&quot;, [validation: :number, kind: :greater_than, number: 12]}], data: #CreateAccount&lt;&gt;, valid?: false&gt;","ref":"Commanded.Command.html","title":"Commanded.Command","type":"behaviour"},{"doc":"","ref":"Commanded.Command.html#field_type/1","title":"Commanded.Command.field_type/1","type":"function"},{"doc":"Optional callback to define validation rules","ref":"Commanded.Command.html#c:handle_validate/1","title":"Commanded.Command.handle_validate/1","type":"callback"},{"doc":"Provides validation before dispatching your commands.","ref":"Commanded.CommandDispatchValidation.html","title":"Commanded.CommandDispatchValidation","type":"module"},{"doc":"Creates a domain event structure. Options from - A struct to adapt the keys from. with - A list of keys to add to the event. drop - A list of keys to drop from the keys adapted from a struct. version - An optional version of the event. Defaults to 1. Example # This is for demonstration purposes only. You don&#39;t need to create a new event to version one. defmodule AccountCreatedVersioned do use Commanded.Event, version: 2, from: CreateAccount, with: [:date, :sex], drop: [:email], defimpl Commanded.Event.Upcaster, for: AccountCreatedWithDroppedKeys do def upcast(%{version: 1} = event, _metadata) do AccountCreatedVersioned.new(event, sex: &quot;maybe&quot;, version: 2) end def upcast(event, _metadata), do: event end end iex&gt; changeset = CreateAccount.new(username: &quot;chris&quot;, email: &quot;chris@example.com&quot;, age: 5) iex&gt; cmd = Ecto.Changeset.apply_changes(changeset) iex&gt; event = AccountCreatedWithDroppedKeys.new(cmd) iex&gt; Commanded.Event.Upcaster.upcast(event, %{}) %AccountCreatedVersioned{age: 5, date: nil, sex: &quot;maybe&quot;, username: &quot;chris&quot;, version: 2}","ref":"Commanded.Event.html","title":"Commanded.Event","type":"module"},{"doc":"Commanded Messaging Common macros for messaging in a Commanded application Commands The Commanded.Command macro creates an Ecto embedded_schema so you can take advantage of the well known Ecto.Changeset API. defmodule BasicCreateAccount do use Commanded.Command, username: :string, email: :string, age: :integer end iex&gt; BasicCreateAccount.new() #Ecto.Changeset&lt;action: nil, changes: %{}, errors: [], data: #BasicCreateAccount&lt;&gt;, valid?: true&gt; Validation defmodule CreateAccount do use Commanded.Command, username: :string, email: :string, age: :integer def handle_validate(command) do command |&gt; validate_required([:username, :email, :age]) |&gt; validate_format(:email, ~r/@/) |&gt; validate_number(:age, greater_than: 12) end end iex&gt; CreateAccount.new() #Ecto.Changeset&lt;action: nil, changes: %{}, errors: [username: {&quot;can&#39;t be blank&quot;, [validation: :required]}, email: {&quot;can&#39;t be blank&quot;, [validation: :required]}, age: {&quot;can&#39;t be blank&quot;, [validation: :required]}], data: #CreateAccount&lt;&gt;, valid?: false&gt; iex&gt; CreateAccount.new(username: &quot;chris&quot;, email: &quot;chris@example.com&quot;, age: 5) #Ecto.Changeset&lt;action: nil, changes: %{age: 5, email: &quot;chris@example.com&quot;, username: &quot;chris&quot;}, errors: [age: {&quot;must be greater than %{number}&quot;, [validation: :number, kind: :greater_than, number: 12]}], data: #CreateAccount&lt;&gt;, valid?: false&gt; To create the actual command struct, use Ecto.Changeset.apply_changes/1 iex&gt; command = CreateAccount.new(username: &quot;chris&quot;, email: &quot;chris@example.com&quot;, age: 5) iex&gt; Ecto.Changeset.apply_changes(command) %CreateAccount{age: 5, email: &quot;chris@example.com&quot;, username: &quot;chris&quot;} Note that apply_changes will not validate values. Events Most events mirror the commands that produce them. So we make it easy to reduce the boilerplate in creating them with the Commanded.Event macro. defmodule BasicAccountCreated do use Commanded.Event, from: CreateAccount end iex&gt; command = CreateAccount.new(username: &quot;chris&quot;, email: &quot;chris@example.com&quot;, age: 5) iex&gt; cmd = Ecto.Changeset.apply_changes(command) iex&gt; BasicAccountCreated.new(cmd) %BasicAccountCreated{ age: 5, email: &quot;chris@example.com&quot;, username: &quot;chris&quot;, version: 1 } Extra Keys There are times when we need keys defined on an event that aren&#39;t part of the originating command. We can add these very easily. defmodule AccountCreatedWithExtraKeys do use Commanded.Event, from: CreateAccount, with: [:date] end iex&gt; command = CreateAccount.new(username: &quot;chris&quot;, email: &quot;chris@example.com&quot;, age: 5) iex&gt; cmd = Ecto.Changeset.apply_changes(command) iex&gt; AccountCreatedWithExtraKeys.new(cmd, date: ~D[2019-07-25]) %AccountCreatedWithExtraKeys{ age: 5, date: ~D[2019-07-25], email: &quot;chris@example.com&quot;, username: &quot;chris&quot;, version: 1 } Excluding Keys And you may also want to drop some keys from your command. defmodule AccountCreatedWithDroppedKeys do use Commanded.Event, from: CreateAccount, with: [:date], drop: [:email] end iex&gt; command = CreateAccount.new(username: &quot;chris&quot;, email: &quot;chris@example.com&quot;, age: 5) iex&gt; cmd = Ecto.Changeset.apply_changes(command) iex&gt; AccountCreatedWithDroppedKeys.new(cmd) %AccountCreatedWithDroppedKeys{ age: 5, date: nil, username: &quot;chris&quot;, version: 1 } Versioning You may have noticed that we provide a default version of 1. You can change the version of an event at anytime. After doing so, you should define an upcast instance that knows how to transform older events into the latest version. # This is for demonstration purposes only. You don&#39;t need to create a new event to version one. defmodule AccountCreatedVersioned do use Commanded.Event, version: 2, from: CreateAccount, with: [:date, :sex], drop: [:email], defimpl Commanded.Event.Upcaster, for: AccountCreatedWithDroppedKeys do def upcast(%{version: 1} = event, _metadata) do AccountCreatedVersioned.new(event, sex: &quot;maybe&quot;, version: 2) end def upcast(event, _metadata), do: event end end iex&gt; command = CreateAccount.new(username: &quot;chris&quot;, email: &quot;chris@example.com&quot;, age: 5) iex&gt; cmd = Ecto.Changeset.apply_changes(command) iex&gt; event = AccountCreatedWithDroppedKeys.new(cmd) iex&gt; Commanded.Event.Upcaster.upcast(event, %{}) %AccountCreatedVersioned{age: 5, date: nil, sex: &quot;maybe&quot;, username: &quot;chris&quot;, version: 2} Note that you won&#39;t normally call upcast manually. Commanded will take care of that for you. Command Dispatch Validation The Commanded.CommandDispatchValidation macro will inject the validate_and_dispatch function into your Commanded.Application.","ref":"CommandedMessaging.html","title":"CommandedMessaging","type":"module"},{"doc":"Commanded Messaging Common macros for messaging in a Commanded application","ref":"readme.html","title":"Commanded Messaging","type":"extras"},{"doc":"This package can be installed by adding commanded_messaging to your list of dependencies in mix.exs: def deps do [ {:commanded_messaging, &quot;~&gt; 0.2.0&quot;} ] end Documentation","ref":"readme.html#installation","title":"Commanded Messaging - Installation","type":"extras"},{"doc":"Commands The Commanded.Command macro creates an Ecto embedded_schema so you can take advantage of the well known Ecto.Changeset API. Default defmodule CreateAccount do use Commanded.Command, username: :string, email: :string, age: :integer end iex&gt; CreateAccount.new() #Ecto.Changeset&lt;action: nil, changes: %{}, errors: [], data: #CreateAccount&lt;&gt;, valid?: true&gt; Validation defmodule CreateAccount do use Commanded.Command, username: :string, email: :string, age: :integer def handle_validate(changeset) do changeset |&gt; validate_required([:username, :email, :age]) |&gt; validate_format(:email, ~r/@/) |&gt; validate_number(:age, greater_than: 12) end end iex&gt; CreateAccount.new() #Ecto.Changeset&lt; action: nil, changes: %{}, errors: [ username: {&quot;can&#39;t be blank&quot;, [validation: :required]}, email: {&quot;can&#39;t be blank&quot;, [validation: :required]}, age: {&quot;can&#39;t be blank&quot;, [validation: :required]} ], data: #CreateAccount&lt;&gt;, valid?: false &gt; iex&gt; changeset = CreateAccount.new(username: &quot;chris&quot;, email: &quot;chris@example.com&quot;, age: 5) #Ecto.Changeset&lt; action: nil, changes: %{age: 5, email: &quot;chris@example.com&quot;, username: &quot;chris&quot;}, errors: [ age: {&quot;must be greater than %{number}&quot;, [validation: :number, kind: :greater_than, number: 12]} ], data: #CreateAccount&lt;&gt;, valid?: false &gt; To create the actual command struct, use Ecto.Changeset.apply_changes/1 iex&gt; cmd = Ecto.Changeset.apply_changes(changeset) %CreateAccount{age: 5, email: &quot;chris@example.com&quot;, username: &quot;chris&quot;} Note that apply_changes will not validate values. Events Most events mirror the commands that produce them. So we make it easy to reduce the boilerplate in creating them with the Commanded.Event macro. defmodule AccountCreated do use Commanded.Event, from: CreateAccount end iex&gt; AccountCreated.new(cmd) %AccountCreated{ age: 5, email: &quot;chris@example.com&quot;, username: &quot;chris&quot;, version: 1 } Extra Keys There are times when we need keys defined on an event that aren&#39;t part of the originating command. We can add these very easily. defmodule AccountCreated do use Commanded.Event, from: CreateAccount, with: [:date] end iex&gt; AccountCreated.new(cmd, date: NaiveDateTime.utc_now()) %AccountCreated{ age: 5, date: ~N[2019-07-25 08:03:15.372212], email: &quot;chris@example.com&quot;, username: &quot;chris&quot;, version: 1 } Excluding Keys And you may also want to drop some keys from your command. defmodule AccountCreated do use Commanded.Event, from: CreateAccount, with: [:date], drop: [:email] end iex&gt; event = AccountCreated.new(cmd) %AccountCreated{age: 5, date: nil, username: &quot;chris&quot;, version: 1} Versioning You may have noticed that we provide a default version of 1. You can change the version of an event at anytime. After doing so, you should define an upcast instance that knows how to transform older events into the latest version. defmodule AccountCreated do use Commanded.Event, version: 2, from: CreateAccount, with: [:date, :sex], drop: [:email] defimpl Commanded.Event.Upcaster do def upcast(%{version: 1} = event, _metadata) do AccountCreated.new(event, sex: &quot;maybe&quot;, version: 2) end def upcast(event, _metadata), do: event end end iex&gt; Commanded.Event.Upcaster.upcast(event, %{}) %AccountCreated{age: 5, date: nil, sex: &quot;maybe&quot;, username: &quot;chris&quot;, version: 2} Note that you won&#39;t normally call upcast manually. Commanded will take care of that for you. Command Dispatch Validation The Commanded.CommandDispatchValidation macro will inject the validate_and_dispatch into your Commanded.Commands.Router. defmodule AccountsRouter do use Commanded.Commands.Router use Commanded.CommandDispatchValidation end iex&gt; AccountsRouter.validate_and_dispatch(changeset) {:error, {:validation_failure, %{age: [&quot;must be greater than 12&quot;]}}} I hope you find this library as useful as my team and I do. -Chris","ref":"readme.html#usage","title":"Commanded Messaging - Usage","type":"extras"}]